@@ -32,7 +32,7 @@
 
 AutoPart::AutoPart(MProcess &mproc, PartMan *pman, Ui::MeInstall &ui, const class QSettings &appConf) noexcept
     : QObject(ui.boxSliderPart), proc(mproc), gui(ui), partman(pman),
-      passCrypto(ui.textCryptoPass, ui.textCryptoPass2, 1, 32, 9, this)
+      passCrypto(ui.textCryptoPass, ui.textCryptoPass2, 1, this)
 {
     checkHibernation = gui.checkHibernationReg;
 
@@ -273,8 +273,8 @@ long long AutoPart::buildLayout(long long rootFormatSize, bool crypto,
         if (remaining > 0) volList->append("/home");
     }
     if (updateTree) {
-        drvitem->addPart(rootFormatSize, "root", crypto);
-        if (remaining > 0) drvitem->addPart(remaining, "home", crypto);
+        drvitem->addPart(rootFormatSize, "/", crypto);
+        if (remaining > 0) drvitem->addPart(remaining, "/home", crypto);
         drvitem->labelParts();
         drvitem->driveAutoSetActive();
     }
@@ -315,7 +315,7 @@ void AutoPart::toggleEncrypt(bool checked) noexcept
     if (!canHibernate) gui.checkHibernationReg->setChecked(false);
 
     setParams(true, checked, gui.checkHibernationReg->isChecked(), true);
-    if (!inBuilder) gui.pushNext->setEnabled(!checked || passCrypto.isValid());
+    if (!inBuilder) gui.pushNext->setEnabled(!checked || passCrypto.valid());
 }
 
 void AutoPart::sliderPressed() noexcept
