@@ -33,7 +33,8 @@ BootMan::BootMan(MProcess &mproc, PartMan &pman, Ui::MeInstall &ui,
     const QSettings &appConf, const QCommandLineParser &appArgs) noexcept
     : QObject(ui.boxMain), proc(mproc), gui(ui), partman(pman)
 {
-    loaderID = appConf.value("PROJECT_SHORTNAME").toString() + appConf.value("VERSION").toString();
+    loaderID = appConf.value("PROJECT_SHORTNAME").toString();
+    loaderLabel = appConf.value("PROJECT_NAME").toString();
     installFromRootDevice = appConf.value("INSTALL_FROM_ROOT_DEVICE").toBool();
     removeNoSplash = appConf.value("REMOVE_NOSPLASH").toBool();
     brave = appArgs.isSet("brave");
@@ -56,16 +57,17 @@ void BootMan::manageConfig(MSettings &config) noexcept
 
 void BootMan::selectBootMain() noexcept
 {
-    const DeviceItem *twit = partman.mounts.value("/boot");
-    if (!twit) twit = partman.mounts.value("/");
+    const auto fit = partman.mounts.find("/boot");
+    const DeviceItem *twit = (fit != partman.mounts.end() ? fit->second : partman.mounts.at("/"));
+
     if (twit->origin) twit = twit->origin;
     while (twit && twit->type != DeviceItem::Partition) twit = twit->parent();
     if (!gui.radioBootPBR->isChecked() && twit) twit = twit->parent();
     if (!twit) return;
     int ixsel = gui.comboBoot->findData(twit->device); // Boot drive or partition
     for(int ixi = 0; ixsel < 0 && ixi < gui.comboBoot->count(); ++ixi) {
-        const QStringList &s = DeviceItem::split(gui.comboBoot->itemData(ixi).toString());
-        if (s.at(0) == twit->device) ixsel = ixi; // Partition on boot drive
+        const DeviceItem::NameParts &s = DeviceItem::split(gui.comboBoot->itemData(ixi).toString());
+        if (s.drive == twit->device) ixsel = ixi; // Partition on boot drive
     }
     if (ixsel >= 0) gui.comboBoot->setCurrentIndex(ixsel);
 }
@@ -96,97 +98,88 @@ void BootMan::buildBootLists() noexcept
 }
 
 // build a grub configuration and install grub
-void BootMan::install()
+void BootMan::install(const QStringList &cmdextra)
 {
-    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
-    if (proc.halted()) return;
+    proc.log(__PRETTY_FUNCTION__, MProcess::LOG_MARKER);
+    MProcess::Section sect(proc, QT_TR_NOOP("GRUB installation failed. You can reboot to"
+        " the live medium and use the GRUB Rescue menu to repair the installation."));
     proc.advance(4, 4);
 
+    sect.setExceptionMode(false);
     // the old initrd is not valid for this hardware
-    proc.shell("/bin/ls /mnt/antiX/boot | grep 'initrd.img-3.6'", nullptr, true);
+    proc.shell("ls /mnt/antiX/boot | grep 'initrd.img-3.6'", nullptr, true);
     const QString &val = proc.readOut();
-    if (!val.isEmpty()) proc.exec("/bin/rm", {"-f", "/mnt/antiX/boot/" + val});
+    if (!val.isEmpty()) proc.exec("rm", {"-f", "/mnt/antiX/boot/" + val});
 
     bool efivars_ismounted = false;
     if (gui.boxBoot->isChecked() && gui.radioBootESP->isChecked()) {
         QString efivars = QStringLiteral("/sys/firmware/efi/efivars");
         if (QFileInfo(efivars).isDir()) {
-            efivars_ismounted = proc.exec("/bin/mountpoint", {"-q", efivars});
+            efivars_ismounted = proc.exec("mountpoint", {"-q", efivars});
         }
-        if (!efivars_ismounted) proc.exec("/bin/mount", {"-t", "efivarfs", "efivarfs", efivars});
+        if (!efivars_ismounted) proc.exec("mount", {"-t", "efivarfs", "efivarfs", efivars});
     }
 
-    if (gui.radioBootESP->isChecked()) mkdir("/mnt/antiX/boot/efi", 0755);
+    sect.setExceptionMode(true);
 
-    // set mounts for chroot
-    proc.exec("/bin/mount", {"--rbind", "--make-rslave", "/dev", "/mnt/antiX/dev"});
-    proc.exec("/bin/mount", {"--rbind", "--make-rslave", "/sys", "/mnt/antiX/sys"});
-    proc.exec("/bin/mount", {"--rbind", "/proc", "/mnt/antiX/proc"});
-    proc.exec("/bin/mount", {"-t", "tmpfs", "-o", "size=100m,nodev,mode=755", "tmpfs", "/mnt/antiX/run"});
-    proc.exec("/bin/mkdir", {"/mnt/antiX/run/udev"});
-    proc.exec("/bin/mount", {"--rbind", "/run/udev", "/mnt/antiX/run/udev"});
-
-    // Trap exceptions here, and re-throw them once the local cleanup is done.
-    const char *failed = nullptr;
-    try {
-        installMain(efivars_ismounted);
-    } catch (const char *msg) {
-        failed = msg;
-    }
-
-    proc.exec("/bin/umount", {"-R", "/mnt/antiX/run"});
-    proc.exec("/bin/umount", {"-R", "/mnt/antiX/proc"});
-    proc.exec("/bin/umount", {"-R", "/mnt/antiX/sys"});
-    proc.exec("/bin/umount", {"-R", "/mnt/antiX/dev"});
-    if (proc.exec("mountpoint", {"-q", "/mnt/antiX/boot/efi"})) {
-        proc.exec("/bin/umount", {"/mnt/antiX/boot/efi"});
-    }
-    if (failed) throw failed;
-}
-void BootMan::installMain(bool efivars_ismounted)
-{
     if (gui.boxBoot->isChecked()) {
         proc.status(tr("Installing GRUB"));
-        const char *failGrub = QT_TR_NOOP("GRUB installation failed. You can reboot to"
-            " the live medium and use the GRUB Rescue menu to repair the installation.");
-        proc.setExceptionMode(failGrub);
 
         // install new Grub now
-        QString arch;
-        const QString &boot = "/dev/" + gui.comboBoot->currentData().toString();
+        const int efisize = proc.detectEFI(true);
+        const QString &bootdev = "/dev/" + gui.comboBoot->currentData().toString();
         if (!gui.radioBootESP->isChecked()) {
             proc.exec("grub-install", {"--target=i386-pc", "--recheck",
-                "--no-floppy", "--force", "--boot-directory=/mnt/antiX/boot", boot});
+                "--no-floppy", "--force", "--boot-directory=/mnt/antiX/boot", bootdev});
         } else {
-            proc.exec("/bin/mount", {boot, "/mnt/antiX/boot/efi"});
+            proc.exec("mount", {"--mkdir", bootdev, "/mnt/antiX/boot/efi"});
             // rename arch to match grub-install target
             proc.exec("cat", {"/sys/firmware/efi/fw_platform_size"}, nullptr, true);
-            arch = proc.readOut();
-            arch = (arch == "32") ? "i386" : "x86_64";  // fix arch name for 32bit
 
             if (efivars_ismounted) {
                 // remove any efivars-dump-entries in NVRAM
-                proc.setExceptionMode(nullptr);
-                proc.shell("/bin/ls /sys/firmware/efi/efivars | grep dump", nullptr, true);
+                sect.setExceptionMode(false);
+                proc.shell("ls /sys/firmware/efi/efivars | grep dump", nullptr, true);
                 const QString &dump = proc.readOut();
-                proc.setExceptionMode(failGrub);
-                if (!dump.isEmpty()) proc.shell("/bin/rm /sys/firmware/efi/efivars/dump*", nullptr, true);
+                sect.setExceptionMode(true);
+                if (!dump.isEmpty()) proc.shell("rm /sys/firmware/efi/efivars/dump*", nullptr, true);
             }
 
-            proc.exec("chroot", {"/mnt/antiX", "grub-install", "--force-extra-removable",
-                "--target=" + arch + "-efi", "--efi-directory=/boot/efi",
-                "--bootloader-id=" + loaderID, "--recheck"});
+            sect.setRoot("/mnt/antiX");
+            proc.exec("grub-install", {"--no-nvram", "--force-extra-removable",
+                (efisize==32 ? "--target=i386-efi" : "--target=x86_64-efi"),
+                "--efi-directory=/boot/efi", "--bootloader-id=" + loaderID, "--recheck"});
+
+            // Update the boot NVRAM variables. Non-critial step so no need to fail.
+            sect.setExceptionMode(false);
+            // Remove old boot variables of the same label.
+            proc.exec("efibootmgr", {}, nullptr, true);
+            const QStringList &existing = proc.readOutLines();
+            const QRegularExpression regex("^Boot([0-9A-F]{4})\\*?\\s(.*)$");
+            for (const QString &entry : existing) {
+                const QRegularExpressionMatch &match = regex.match(entry);
+                if (match.hasMatch() && match.captured(2) == loaderLabel) {
+                    proc.exec("efibootmgr", {"-qBb", match.captured(1)});
+                }
+            }
+            // Add a new NVRAM boot variable.
+            const DeviceItem::NameParts &bs = DeviceItem::split(bootdev);
+            proc.exec("efibootmgr", {"-qcL", loaderLabel, "-d", "/dev/"+bs.drive, "-p", bs.partition,
+                "-l", "/EFI/" + loaderID + (efisize==32 ? "/grubia32.efi" : "/grubx64.efi")});
+            sect.setExceptionMode(true);
+            sect.setRoot(nullptr);
         }
 
         //get /etc/default/grub codes
-        QSettings grubSettings("/etc/default/grub", QSettings::NativeFormat);
-        QString grubDefault=grubSettings.value("GRUB_CMDLINE_LINUX_DEFAULT").toString();
+        const QSettings grubSettings("/etc/default/grub", QSettings::NativeFormat);
+        const QString &grubDefault = grubSettings.value("GRUB_CMDLINE_LINUX_DEFAULT").toString();
         qDebug() << "grubDefault is " << grubDefault;
 
         //added non-live boot codes to those in /etc/default/grub, remove duplicates
         proc.shell("/live/bin/non-live-cmdline", nullptr, true); // Get non-live boot codes
         QStringList finalcmdline = proc.readOut().split(" ");
         finalcmdline.append(grubDefault.split(" "));
+        finalcmdline.append(cmdextra);
         qDebug() << "intermediate" << finalcmdline;
 
         //get built-in config_cmdline
@@ -229,36 +222,27 @@ void BootMan::installMain(bool efivars_ismounted)
         proc.shell(cmd.arg(finalcmdlinestring));
 
         //copy memtest efi files if needed
-
-        if (proc.detectEFI()) {
-            mkdir("/mnt/antiX/boot/uefi-mt", 0755);
-            QString mtest;
-            QString mtest_dev;
-            QString mtest_ram;
-            if (arch == "i386") {
-                mtest_dev = "/live/boot-dev/boot/uefi-mt/mtest-32.efi";
-                mtest_ram = "/live/to-ram/boot/uefi-mt/mtest-32.efi";
-            } else {
-                mtest_dev = "/live/boot-dev/boot/uefi-mt/mtest-64.efi";
-                mtest_ram = "/live/to-ram/boot/uefi-mt/mtest-64.efi";
+        if (efisize) {
+            QString mtest = QString("/live/to-ram/boot/uefi-mt/mtest-%1.efi").arg(efisize);
+            if (!QFileInfo::exists(mtest)) {
+                mtest = QString("/live/boot-dev/boot/uefi-mt/mtest-%1.efi").arg(efisize);
+                if (!QFileInfo::exists(mtest)) mtest.clear();
             }
-            if (QFileInfo::exists(mtest_ram)) {
-                mtest = mtest_ram;
-            } else if (QFileInfo::exists(mtest_dev)) {
-                mtest = mtest_dev;
-            }
-            if (!mtest.isNull()) {
-                proc.exec("/bin/cp", {mtest, "/mnt/antiX/boot/uefi-mt"});
+            if (!mtest.isEmpty()) {
+                proc.mkpath("/mnt/antiX/boot/uefi-mt", 0755);
+                proc.exec("cp", {mtest, "/mnt/antiX/boot/uefi-mt"});
             }
         }
         proc.status();
 
+        sect.setRoot("/mnt/antiX");
         //update grub with new config
-        proc.exec("chroot", {"/mnt/antiX", "update-grub"});
+        proc.exec("update-grub");
 
         if (!gui.radioBootESP->isChecked()) {
-            /* Prevent debconf pestering the user when GRUB gets updated. */
-            proc.exec("udevadm", {"info", boot}, nullptr, true);
+            /* Prevent debconf pestering the user when GRUB gets updated. Non-critical. */
+            MProcess::Section(proc, nullptr);
+            proc.exec("udevadm", {"info", bootdev}, nullptr, true);
             const QStringList &lines = proc.readOutLines();
             /* Obtain the best ID to use for the disk or partition. */
             QByteArray diskpath;
@@ -272,27 +256,27 @@ void BootMan::installMain(bool efivars_ismounted)
                     }
                 }
             }
-            if (diskpath.isEmpty()) throw failGrub;
-            /* Setup debconf to achieve the objective of silence. */
-            diskpath.prepend("grub-pc grub-pc/install_devices multiselect /dev/");
-            proc.exec("chroot", {"/mnt/antiX", "debconf-set-selections"}, &diskpath);
+            if (!diskpath.isEmpty()) {
+                /* Setup debconf to achieve the objective of silence. */
+                diskpath.prepend("grub-pc grub-pc/install_devices multiselect /dev/");
+                proc.exec("debconf-set-selections", {}, &diskpath);
+            }
         }
     }
 
     proc.status(tr("Updating initramfs"));
-    proc.setExceptionMode(QT_TR_NOOP("Failed to update initramfs."));
+    sect.setExceptionMode(QT_TR_NOOP("Failed to update initramfs."));
     //if useing f2fs, then add modules to /etc/initramfs-tools/modules
     //if (rootTypeCombo->currentText() == "f2fs" || homeTypeCombo->currentText() == "f2fs") {
-        //proc.shell("grep -q f2fs /mnt/antiX/etc/initramfs-tools/modules || echo f2fs >> /mnt/antiX/etc/initramfs-tools/modules");
-        //proc.shell("grep -q crypto-crc32 /mnt/antiX/etc/initramfs-tools/modules || echo crypto-crc32 >> /mnt/antiX/etc/initramfs-tools/modules");
+        //proc.shell("grep -q f2fs /etc/initramfs-tools/modules || echo f2fs >> /etc/initramfs-tools/modules");
+        //proc.shell("grep -q crypto-crc32 /etc/initramfs-tools/modules || echo crypto-crc32 >> /etc/initramfs-tools/modules");
     //}
 
     // Use MODULES=dep to trim the initrd, often results in faster boot.
-    proc.exec("sed", {"-i", "-r", "s/MODULES=.*/MODULES=dep/", "/mnt/antiX/etc/initramfs-tools/initramfs.conf"});
+    proc.exec("sed", {"-i", "-r", "s/MODULES=.*/MODULES=dep/", "/etc/initramfs-tools/initramfs.conf"});
 
-    proc.exec("chroot", {"/mnt/antiX", "update-initramfs", "-u", "-t", "-k", "all"});
+    proc.exec("update-initramfs", {"-u", "-t", "-k", "all"});
     proc.status();
-    proc.setExceptionMode(nullptr);
 }
 
 /* Slots */
