@@ -22,6 +22,7 @@
 #include <dirent.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/sysinfo.h>
 
 #include <QCommandLineParser>
 #include <QSettings>
@@ -103,13 +104,14 @@ MInstall::MInstall(QSettings &acfg, const QCommandLineParser &args, const QStrin
             startup();
             phase = Ready;
         } catch (const char *msg) {
-            if(msg[0]) {
-                proc.log(QStringLiteral("FAILED START - ") + msg, MProcess::Fail);
-                QMessageBox::critical(this, windowTitle(), tr(msg));
-            }
             proc.unhalt();
             setupAutoMount(true);
-            exit(EXIT_FAILURE);
+            const bool closenow = (!msg || !*msg);
+            if(!closenow) {
+                proc.log(QStringLiteral("FAILED START - ") + msg, MProcess::LOG_FAIL);
+                labelSplash->setText(tr(msg));
+            }
+            abortEndUI(closenow);
         }
     });
 }
@@ -128,7 +130,7 @@ MInstall::~MInstall() {
 // meant to be run after the installer becomes visible
 void MInstall::startup()
 {
-    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
+    proc.log(__PRETTY_FUNCTION__, MProcess::LOG_MARKER);
     connect(pushClose, &QPushButton::clicked, this, &MInstall::close);
 
     if (!modeOOBE) {
@@ -162,6 +164,8 @@ void MInstall::startup()
             checkmd5 = nullptr;
         }
 
+        setupZRam(); // Start zram swap. In particular, the Argon2id KDF for LUKS uses a lot of memory.
+
         partman = new PartMan(proc, *this, appConf, appArgs);
         base = new Base(proc, *partman, *this, appConf, appArgs);
         bootman = new BootMan(proc, *partman, *this, appConf, appArgs);
@@ -183,7 +187,7 @@ void MInstall::startup()
                             + "\n" + link_block);
 
         // Password box setup
-        passCryptoCust = new PassEdit(textCryptoPassCust, textCryptoPassCust2, 1, 32, 9, this);
+        passCryptoCust = new PassEdit(textCryptoPassCust, textCryptoPassCust2, 1, this);
         connect(passCryptoCust, &PassEdit::validationChanged, pushNext, &QPushButton::setEnabled);
     }
 
@@ -245,10 +249,23 @@ void MInstall::splashSetThrobber(bool active) noexcept
     labelSplash->update();
 }
 
+void MInstall::setupZRam()
+{
+    struct sysinfo sinfo;
+    if (sysinfo(&sinfo) != 0) return;
+    const long long zrsize = (long long)sinfo.totalram * sinfo.mem_unit;
+    // Reported compressed data can be inaccurate if zswap is enabled, especially on Liquorix.
+    proc.shell("echo 0 > /sys/module/zswap/parameters/enabled");
+    proc.exec("modprobe", {"zram"});
+    if (!proc.exec("zramctl", {"zram0", "-a","zstd", "-s", QString::number(zrsize)})) return;
+    if (!proc.exec("mkswap", {"-q", "/dev/zram0"})) return;
+    proc.exec("swapon", {"-p","32767", "/dev/zram0"});
+}
+
 // turn auto-mount off and on
 void MInstall::setupAutoMount(bool enabled)
 {
-    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
+    proc.log(__PRETTY_FUNCTION__, MProcess::LOG_MARKER);
 
     if (autoMountEnabled == enabled) return;
     // check if the systemctl program is present
@@ -352,9 +369,6 @@ bool MInstall::processNextPhase() noexcept
             autoMountEnabled = true; // disable auto mount by force
             if (!pretend) setupAutoMount(false);
 
-            // cleanup previous mounts
-            cleanup(false);
-
             // the core of the installation
             phase = Installing;
             if (!pretend) {
@@ -367,7 +381,9 @@ bool MInstall::processNextPhase() noexcept
             }
             if (widgetStack->currentWidget() != pageProgress) {
                 progInstall->setEnabled(false);
-                proc.status(tr("Paused for required operator input"));
+                // Using proc.status() prepends the percentage to the text.
+                progInstall->setFormat(tr("Paused for required operator input"));
+                proc.log(progInstall->format(), MProcess::LOG_STATUS);
                 QApplication::beep();
             }
             phase = WaitingForInfo;
@@ -382,9 +398,10 @@ bool MInstall::processNextPhase() noexcept
                 if (oem) oobe->enable();
                 oobe->process();
                 manageConfig(ConfigSave);
-                proc.exec("/bin/sync"); // the sync(2) system call will block the GUI
-                swapman->install();
-                bootman->install();
+                proc.exec("sync"); // the sync(2) system call will block the GUI
+                QStringList grubextra;
+                swapman->install(grubextra);
+                bootman->install(grubextra);
             } else if (!pretendToInstall(5, 100)) {
                 throw "";
             }
@@ -419,8 +436,7 @@ bool MInstall::processNextPhase() noexcept
         if (!msg || !msg[0] || abortion) {
             msg = QT_TR_NOOP("The installation was aborted.");
         }
-        proc.log("FAILED Phase " + QString::number(phase) + " - " + msg, MProcess::Fail);
-        proc.setExceptionMode(nullptr);
+        proc.log("FAILED Phase " + QString::number(phase) + " - " + msg, MProcess::LOG_FAIL);
 
         const bool closing = (abortion == Closing);
         labelSplash->setText(tr(msg));
@@ -432,16 +448,7 @@ bool MInstall::processNextPhase() noexcept
         }
 
         abortion = Aborted;
-        if (closing) this->close();
-        else {
-            splashSetThrobber(false);
-            boxMain->unsetCursor();
-            // Close should be the right button at this stage.
-            disconnect(pushNext);
-            connect(pushNext, &QPushButton::clicked, this, &MInstall::close);
-            pushNext->setText(pushClose->text());
-            pushNext->show();
-        }
+        abortEndUI(closing);
         return false;
     }
 
@@ -489,7 +496,6 @@ void MInstall::manageConfig(enum ConfigAction mode) noexcept
                 textCryptoPassCust->setText(epass);
                 textCryptoPassCust2->setText(epass);
             }
-            partman->loadKeyMaterial(config->value("KeyMaterial").toString());
         }
         config->endGroup();
     }
@@ -509,7 +515,6 @@ void MInstall::manageConfig(enum ConfigAction mode) noexcept
         config->sync();
         QFile::remove("/etc/minstalled.conf");
         QFile::copy(config->fileName(), "/etc/minstalled.conf");
-        config->dumpDebug();
     }
 
     if (config->bad) {
@@ -655,7 +660,7 @@ void MInstall::pageDisplayed(int next) noexcept
             + tr("If you need more control over where %1 is installed to, select \"<b>%2</b>\" and click <b>Next</b>."
                 " On the next page, you will then be able to select and configure the storage devices and"
                 " partitions you need.").arg(PROJECTNAME, radioCustomPart->text().remove('&')) + "</p>");
-        enableNext = radioCustomPart->isChecked() || !boxEncryptAuto->isChecked() || autopart->passCrypto.isValid();
+        enableNext = radioCustomPart->isChecked() || !boxEncryptAuto->isChecked() || autopart->passCrypto.valid();
         break;
 
     case Step::Partitions:
@@ -738,7 +743,7 @@ void MInstall::pageDisplayed(int next) noexcept
             "<p><b>" + tr("Virtual Devices") + "</b><br/>"
             + tr("If the intaller detects any virtual devices such as opened LUKS partitions, LVM logical volumes or software-based RAID volumes, they may be used for the installation.") + "</p>"
             "<p>" + tr("The use of virtual devices (beyond preserving encrypted file systems) is an advanced feature. You may have to edit some files (eg. initramfs, crypttab, fstab) to ensure the virtual devices used are created upon boot.") + "</p>");
-        enableNext = !(boxCryptoPass->isEnabledTo(boxCryptoPass->parentWidget())) || passCryptoCust->isValid();
+        enableNext = !(boxCryptoPass->isEnabledTo(boxCryptoPass->parentWidget())) || passCryptoCust->valid();
         break;
 
     case Step::Boot: // Start of installation.
@@ -1049,7 +1054,7 @@ void MInstall::abortUI(bool manual, bool closing) noexcept
                 tr("The installation and configuration is incomplete.\nDo you really want to stop now?"),
                 QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
             if (rc == QMessageBox::No) return;
-            proc.log("MANUALLY ABORTED", MProcess::Fail);
+            proc.log("MANUALLY ABORTED", MProcess::LOG_FAIL);
         }
     }
     // At this point the abortion has not been cancelled.
@@ -1059,26 +1064,32 @@ void MInstall::abortUI(bool manual, bool closing) noexcept
     // Early phase bump if waiting on input to trigger abortion cleanup.
     if (manual && phase == WaitingForInfo) processNextPhase();
 }
+void MInstall::abortEndUI(bool closenow) noexcept
+{
+    if (closenow) qApp->exit(EXIT_FAILURE); // this->close() doesn't work.
+    else {
+        splashSetThrobber(false);
+        boxMain->unsetCursor();
+        // Close should be the right button at this stage.
+        disconnect(pushNext);
+        connect(pushNext, &QPushButton::clicked, this, &MInstall::close);
+        pushNext->setText(pushClose->text());
+        pushNext->show();
+    }
+}
 
 // run before closing the app, do some cleanup
-void MInstall::cleanup(bool endclean)
+void MInstall::cleanup()
 {
-    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
+    proc.log(__PRETTY_FUNCTION__, MProcess::LOG_MARKER);
     if (pretend) return;
 
-    if (endclean) {
-        proc.exec("/usr/bin/cp", {"/var/log/minstall.log", "/mnt/antiX/var/log"});
-        proc.exec("/usr/bin/rm", {"-rf", "/mnt/antiX/mnt/antiX"});
-    }
-    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/boot/efi"})) proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/boot/efi"});
-    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/proc"}))     proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/proc"});
-    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/sys"}))      proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/sys"});
-    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/dev/shm"}))  proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/dev/shm"});
-    if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/dev"}))      proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/dev"});
-    if (endclean) {
-        if (!mountkeep) partman->unmount();
-        setupAutoMount(true);
-    }
+    if (config) config->dumpDebug();
+    else qDebug() << "NO CONFIG";
+    proc.exec("cp", {"/var/log/minstall.log", "/mnt/antiX/var/log"});
+    proc.exec("rm", {"-rf", "/mnt/antiX/mnt/antiX"});
+    if (!mountkeep) partman->clearWorkArea();
+    setupAutoMount(true);
 }
 
 void MInstall::on_progInstall_valueChanged(int value) noexcept
@@ -1176,5 +1187,5 @@ void MInstall::on_pushSetKeyboard_clicked() noexcept
 void MInstall::on_radioEntireDisk_toggled(bool checked) noexcept
 {
     boxAutoPart->setEnabled(checked);
-    pushNext->setEnabled(!checked || !boxEncryptAuto->isChecked() || autopart->passCrypto.isValid());
+    pushNext->setEnabled(!checked || !boxEncryptAuto->isChecked() || autopart->passCrypto.valid());
 }
