@@ -1,6 +1,5 @@
 /***************************************************************************
  * MProcess class - Installer-specific extensions to QProcess.
- ***************************************************************************
  *
  *   Copyright (C) 2019 by AK-47
  *   Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,13 +17,17 @@
  * This file is part of the gazelle-installer.
  ***************************************************************************/
 
+#include <unistd.h>
+#include <sys/stat.h>
+
 #include <QApplication>
 #include <QProcessEnvironment>
 #include <QDebug>
 #include <QEventLoop>
 #include <QTimer>
 #include <QDir>
-#include <unistd.h>
+#include <QListWidget>
+#include <QProgressBar>
 
 #include "mprocess.h"
 
@@ -37,7 +40,7 @@ MProcess::MProcess(QObject *parent)
     setProcessEnvironment(env);
 }
 
-void MProcess::setupUI(QListWidget *listLog, QProgressBar *progInstall)
+void MProcess::setupUI(QListWidget *listLog, QProgressBar *progInstall) noexcept
 {
     logView = listLog;
     progBar = progInstall;
@@ -47,17 +50,19 @@ void MProcess::setupUI(QListWidget *listLog, QProgressBar *progInstall)
     listLog->setPalette(pal);
 }
 
-void MProcess::setupChildProcess()
+void MProcess::setupChildProcess() noexcept
 {
-    if (chRoot.isEmpty()) return;
-    chroot(chRoot.toUtf8().constData());
-    chdir("/");
+    if (section && section->rootdir) {
+        if (chroot(section->rootdir) != 0) exit(255);
+        if (chdir("/") != 0) exit(255);
+    }
 }
-void MProcess::setChRoot(const QString &newroot)
+
+inline bool MProcess::checkHalt()
 {
-    if (!newroot.isEmpty()) log("Set chroot: " + newroot, Standard);
-    else if (!chRoot.isEmpty() && newroot.isEmpty()) log("End chroot: " + chRoot, Standard);
-    chRoot = newroot;
+    if (halting == ThrowHalt) throw("");
+    else if (halting == Halted) return true;
+    return false;
 }
 
 bool MProcess::exec(const QString &program, const QStringList &arguments,
@@ -74,8 +79,7 @@ bool MProcess::exec(const QString &program, const QStringList &arguments,
     if (input && !(input->isEmpty())) write(*input);
     closeWriteChannel();
     eloop.exec();
-    disconnect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), nullptr, nullptr);
-    disconnect(this, QOverload<QProcess::ProcessError>::of(&QProcess::errorOccurred), nullptr, nullptr);
+    disconnect(&eloop);
 
     if (debugUnusedOutput) {
         bool hasOut = false;
@@ -98,55 +102,69 @@ bool MProcess::exec(const QString &program, const QStringList &arguments,
         }
     }
 
-    int status = 1;
-    if (exitStatus() != QProcess::NormalExit) status = -1;
-    else if (error() != QProcess::UnknownError) status = -2;
-    else if (exitCode() != 0) status = 0;
+    enum Status status = STATUS_OK;
+    if (exitStatus() != QProcess::NormalExit) status = STATUS_CRITICAL; // Process crash
+    else if (error() != QProcess::UnknownError) status = STATUS_CRITICAL; // Start error
+    else if (exitCode() != 0) status = STATUS_ERROR; // Process error
 
     QDebug dbg(qDebug().nospace() << "Exit #" << execount << ": " << exitCode());
     if (status < 0) dbg << " " << exitStatus() << " " << error();
 
     log(logEntry, status);
-    return (status > 0);
+    if (status != STATUS_OK) {
+        if (section && section->failmsg && (status==STATUS_CRITICAL || section->strictfail)) {
+            throw section->failmsg;
+        }
+        return false;
+    }
+    return true;
 }
 
 bool MProcess::exec(const QString &program, const QStringList &arguments, const QByteArray *input, bool needRead)
 {
-    if (halting) return false;
+    if (checkHalt()) return false;
     ++execount;
     const QString &cmd = joinCommand(program, arguments);
     qDebug().nospace().noquote() << "Exec #" << execount << ": " << cmd;
-    return exec(program, arguments, input, needRead, log(cmd, Exec));
+    return exec(program, arguments, input, needRead, log(cmd, LOG_EXEC));
 }
 bool MProcess::shell(const QString &cmd,  const QByteArray *input, bool needRead)
 {
-    if (halting) return false;
+    if (checkHalt()) return false;
     ++execount;
     qDebug().nospace().noquote() << "Bash #" << execount << ": " << cmd;
-    return exec("/bin/bash", {"-c", cmd}, input, needRead, log(cmd, Exec));
+    return exec("/usr/bin/bash", {"-c", cmd}, input, needRead, log(cmd, LOG_EXEC));
 }
 
-QString MProcess::readOut(bool everything)
+QString MProcess::readOut(bool everything) noexcept
 {
     QString strout(readAllStandardOutput().trimmed());
     if (everything) return strout;
     return strout.section("\n", 0, 0);
 }
-QStringList MProcess::readOutLines()
+QStringList MProcess::readOutLines() noexcept
 {
     return QString(readAllStandardOutput().trimmed()).split('\n', Qt::SkipEmptyParts);
 }
 
-void MProcess::halt()
+void MProcess::halt(bool exception) noexcept
 {
-    halting = true;
-    terminate();
-    QTimer::singleShot(5000, this, &QProcess::kill);
-    setChRoot();
+    log(__PRETTY_FUNCTION__, LOG_MARKER);
+    halting = exception ? ThrowHalt : Halted;
+    if(state() != QProcess::NotRunning) {
+        QEventLoop eloop;
+        connect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), &eloop, &QEventLoop::quit);
+        closeReadChannel(QProcess::StandardOutput);
+        closeReadChannel(QProcess::StandardError);
+        closeWriteChannel();
+        this->terminate();
+        QTimer::singleShot(5000, this, &QProcess::kill);
+        eloop.exec();
+    }
 }
-
-void MProcess::unhalt()
+void MProcess::unhalt() noexcept
 {
+    if (halting == NoHalt) return;
     QEventLoop eloop;
     connect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), &eloop, &QEventLoop::quit);
     if (state() != QProcess::NotRunning) {
@@ -155,11 +173,10 @@ void MProcess::unhalt()
         closeWriteChannel();
         eloop.exec();
     }
-    disconnect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), nullptr, nullptr);
-    halting = false;
+    halting = NoHalt;
 }
 
-QString MProcess::joinCommand(const QString &program, const QStringList &arguments)
+QString MProcess::joinCommand(const QString &program, const QStringList &arguments) noexcept
 {
     QString text = program;
     for(QString arg : arguments) {
@@ -177,46 +194,49 @@ QString MProcess::joinCommand(const QString &program, const QStringList &argumen
     return text;
 }
 
-QListWidgetItem *MProcess::log(const QString &text, const LogType type)
+QListWidgetItem *MProcess::log(const QString &text, const LogType type) noexcept
 {
-    if (type == Standard) qDebug().noquote() << text;
-    else if (type == Section) qDebug().noquote() << "<<" << text << ">>";
-    else if (type == Status) qDebug().noquote() << "++" << text << "++";
-    else if (type == Fail) qDebug().noquote() << "--" << text << "--";
-    if (!logView || type == Section) return nullptr;
+    if (type == LOG_LOG) qDebug().noquote() << text;
+    else if (type == LOG_MARKER) qDebug().noquote() << "<<" << text << ">>";
+    else if (type == LOG_STATUS) qDebug().noquote() << "++" << text << "++";
+    else if (type == LOG_FAIL) qDebug().noquote() << "--" << text << "--";
+    if (!logView || type == LOG_MARKER) return nullptr;
     QListWidgetItem *entry = new QListWidgetItem(text, logView);
     logView->addItem(entry);
-    if (type == Exec) entry->setForeground(Qt::cyan);
-    else if (type == Status || type == Fail) {
+    if (type == LOG_EXEC) entry->setForeground(Qt::cyan);
+    else if (type == LOG_STATUS || type == LOG_FAIL) {
         QFont font(entry->font());
         font.setBold(true);
         entry->setFont(font);
-        if (type == Fail) entry->setForeground(Qt::magenta);
+        if (type == LOG_FAIL) entry->setForeground(Qt::magenta);
     }
     logView->scrollToBottom();
     return entry;
 }
-void MProcess::log(QListWidgetItem *entry, const int status)
+void MProcess::log(QListWidgetItem *entry, enum Status status) noexcept
 {
     if (!entry) return;
-    if (status > 0) entry->setForeground(Qt::green);
-    else if (status < 0) entry->setForeground(Qt::red);
-    else entry->setForeground(Qt::yellow);
+    switch (status) {
+        case STATUS_CRITICAL: entry->setForeground(Qt::red); break;
+        case STATUS_OK: entry->setForeground(Qt::green); break;
+        case STATUS_ERROR: entry->setForeground(Qt::yellow); break;
+        default: entry->setForeground(Qt::blue); break;
+    }
 }
 
-void MProcess::status(const QString &text, long progress)
+void MProcess::status(const QString &text, long progress) noexcept
 {
-    if (!progBar) log(text, Status);
+    if (!progBar) log(text, LOG_STATUS);
     else {
         QString fmt = "%p% - " + text;
         if (progBar->format() != fmt) {
-            log(text, Status);
+            log(text, LOG_STATUS);
             progBar->setFormat(fmt);
         }
         status(progress);
     }
 }
-void MProcess::status(long progress)
+void MProcess::status(long progress) noexcept
 {
     if (progress < 0) ++progSlicePos;
     else progSlicePos = progress;
@@ -228,7 +248,7 @@ void MProcess::status(long progress)
     if (progBar) progBar->setValue(progSliceStart + slice);
     qApp->processEvents();
 }
-void MProcess::advance(int space, long steps)
+void MProcess::advance(int space, long steps) noexcept
 {
     if (space < 0) steps = progSliceSpace = progSliceStart = space = 0; // Reset progress
     progSliceStart += progSliceSpace;
@@ -241,13 +261,12 @@ void MProcess::advance(int space, long steps)
 
 // Common functions that are traditionally carried out by processes.
 
-void MProcess::sleep(const int msec, const bool silent)
+void MProcess::sleep(const int msec, const bool silent) noexcept
 {
-    if (halting) return;
     QListWidgetItem *logEntry = nullptr;
     if (!silent) {
         ++sleepcount;
-        logEntry = log(QString("SLEEP: %1ms").arg(msec), Exec);
+        logEntry = log(QString("SLEEP: %1ms").arg(msec), LOG_EXEC);
         qDebug().nospace() << "Sleep #" << sleepcount << ": " << msec << "ms";
     }
     QTimer cstimer(this);
@@ -257,15 +276,20 @@ void MProcess::sleep(const int msec, const bool silent)
     const int rc = eloop.exec();
     if (!silent) {
         qDebug().nospace() << "Sleep #" << sleepcount << ": exit " << rc;
-        log(logEntry, rc == 0 ? 1 : -1);
+        log(logEntry, rc == 0 ? STATUS_OK : STATUS_CRITICAL);
     }
 }
-bool MProcess::mkpath(const QString &path)
+bool MProcess::mkpath(const QString &path, mode_t mode, bool force)
 {
-    QListWidgetItem *logEntry = log("MKPATH: "+path, Exec);
-    const bool rc = QDir().mkpath(path);
+    if (checkHalt()) return false;
+    if (!force && QFileInfo::exists(path)) return true;
+
+    QListWidgetItem *logEntry = log("MKPATH: "+path, LOG_EXEC);
+    bool rc = QDir().mkpath(path);
+    if (mode && rc) rc = (chmod(path.toUtf8().constData(), mode) == 0);
     qDebug() << (rc ? "MkPath(SUCCESS):" : "MkPath(FAILURE):") << path;
-    log(logEntry, rc ? 1 : -1);
+    log(logEntry, rc ? STATUS_OK : STATUS_CRITICAL);
+    if(!rc && section && section->failmsg) throw section->failmsg;
     return rc;
 }
 
@@ -306,3 +330,29 @@ bool MProcess::detectMac()
     }
     return rc;
 }
+
+// Local execution environment
+
+MProcess::Section::Section(class MProcess &mproc) noexcept
+    : proc(mproc), oldsection(mproc.section)
+{
+    if (oldsection) {
+        failmsg = oldsection->failmsg;
+        rootdir = oldsection->rootdir;
+        strictfail = oldsection->strictfail;
+    }
+    mproc.section = this;
+}
+MProcess::Section::~Section() noexcept
+{
+    const char *oldroot = oldsection ? oldsection->rootdir : nullptr;
+    if (qstrcmp(oldroot, rootdir)) proc.log(QString("Revert root: ")+oldroot, LOG_LOG);
+    proc.section = oldsection;
+}
+
+// Note: newroot must be valid and unchanged for the life of the chroot.
+void MProcess::Section::setRoot(const char *newroot) noexcept
+{
+    if (qstrcmp(newroot, rootdir)) proc.log(QString("Change root: ")+newroot, LOG_LOG);
+    rootdir = newroot; // Might be different pointers to the same text.
+}
