@@ -19,15 +19,15 @@
 #ifndef PARTMAN_H
 #define PARTMAN_H
 
+#include <map>
+#include <stack>
+#include <vector>
 #include <QCommandLineParser>
 #include <QAbstractItemModel>
 #include <QStyledItemDelegate>
 #include <QString>
-#include <QMap>
-#include <QStack>
 
 #include "ui_meinstall.h"
-#include "safecache.h"
 
 #define KB 1024LL
 #define MB 1048576LL
@@ -45,7 +45,7 @@ class DeviceItem
     friend class PartMan;
     friend class DeviceItemIterator;
     friend class DeviceItemDelegate;
-    QVector<DeviceItem *> children;
+    std::vector<DeviceItem *> children;
     DeviceItem *parentItem = nullptr;
     class PartMan *partman = nullptr;
     int order = -1;
@@ -93,7 +93,7 @@ public:
     int row() const noexcept;
     DeviceItem *parent() const noexcept;
     DeviceItem *child(int row) const noexcept;
-    int indexOfChild(DeviceItem *child) noexcept;
+    int indexOfChild(const DeviceItem *child) const noexcept;
     int childCount() const noexcept;
     void sortChildren() noexcept;
     // Layout finishing
@@ -120,19 +120,24 @@ public:
     QStringList allowedFormats() const noexcept;
     QString finalFormat() const noexcept;
     QString shownFormat(const QString &fmt) const noexcept;
+    inline QString shownFormat() const noexcept { return shownFormat(format); }
     inline bool isVolume() const noexcept { return (type == Partition || type == VirtualBD); }
     bool canMount(bool pointonly = true) const noexcept;
     long long driveFreeSpace(bool inclusive = false) const noexcept;
     /* Convenience */
+    struct NameParts {
+        QString drive;
+        QString partition;
+    };
     void addToCombo(QComboBox *combo, bool warnNasty = false) const noexcept;
-    static QStringList split(const QString &devname) noexcept;
+    static NameParts split(const QString &devname) noexcept;
     static QString join(const QString &drive, int partnum) noexcept;
 };
 class DeviceItemIterator
 {
     DeviceItem *pos = nullptr;
     int ixPos = 0;
-    QStack<int> ixParents;
+    std::stack<int, std::vector<int>> ixParents;
 public:
     DeviceItemIterator(DeviceItem *item) noexcept : pos(item) {}
     DeviceItemIterator(const PartMan &partman) noexcept;
@@ -162,14 +167,13 @@ class PartMan : public QAbstractItemModel
     DeviceItem root;
     DeviceItem *changing = nullptr;
     Ui::MeInstall &gui;
-    SafeCache key;
     bool brave, gptoverride;
     void scanVirtualDevices(bool rescan);
     void resizeColumnsToFit() noexcept;
     void preparePartitions();
     void formatPartitions();
     void prepareSubvolumes(DeviceItem *partit);
-    bool fixCryptoSetup();
+    void fixCryptoSetup();
     bool makeFstab();
     void mountPartitions();
     void treeItemChange() noexcept;
@@ -184,7 +188,9 @@ class PartMan : public QAbstractItemModel
     void partMenuUnlock(DeviceItem *twit);
     void partMenuLock(DeviceItem *twit);
     void scanSubvolumes(DeviceItem *partit);
-    void luksFormat(DeviceItem *partit, const QByteArray &password);
+    bool confirmSpace(class QMessageBox &msgbox) noexcept;
+    bool confirmBootable(class QMessageBox &msgbox) noexcept;
+    void luksFormat();
     void luksOpen(DeviceItem *partit, const QString &luksfs, const QByteArray &password);
 public:
     enum TreeColumns {
@@ -207,9 +213,9 @@ public:
         long long minimum = 0;
         long long preferred = 0;
     };
-    QMap<QString, struct VolumeSpec> volSpecs;
+    std::map<QString, struct VolumeSpec> volSpecs;
     QString bootUUID;
-    QMap<QString, DeviceItem *> mounts;
+    std::map<QString, DeviceItem *> mounts;
     class AutoPart *autopart = nullptr;
     PartMan(class MProcess &mproc, class Ui::MeInstall &ui,
         const class QSettings &appConf, const QCommandLineParser &appArgs);
@@ -222,17 +228,13 @@ public:
     int countPrepSteps() noexcept;
     void prepStorage();
     void installTabs();
-    void loadKeyMaterial(const QString &keyfile);
-    void unmount();
+    void clearWorkArea();
     int swapCount() const noexcept;
     int isEncrypt(const QString &point) const noexcept;
     DeviceItem *findByPath(const QString &devpath) const noexcept;
     DeviceItem *findHostDev(const QString &path) const noexcept;
     struct VolumeSpec volSpecTotal(const QString &path, const QStringList &vols) const noexcept;
-    inline struct VolumeSpec volSpecTotal(const QString &path) const noexcept
-    {
-        return volSpecTotal(path, mounts.keys());
-    }
+    struct VolumeSpec volSpecTotal(const QString &path) const noexcept;
     // Model View Controller
     QVariant data(const QModelIndex &index, int role) const noexcept override;
     bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) noexcept override;
